"""Base class for integer values."""
from taurus.entity.value.base_value import BaseValue


class IntegerValue(BaseValue):
    """A base class for values that correspond to a distribution over the integers."""
