from taurus.entity.template.attribute_template import AttributeTemplate


class ParameterTemplate(AttributeTemplate):
    """A template for the parameter attribute."""

    typ = "parameter_template"
